/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package jdk.nashorn.api.tree;

import jdk.nashorn.internal.ir.LiteralNode;

final class LiteralTreeImpl extends ExpressionTreeImpl implements LiteralTree {
    private final Object value;
    private final Kind kind;
    LiteralTreeImpl(final LiteralNode<?> node) {
        super(node);
        this.kind = literalKind(node);
        this.value = node.getValue();
    }

    @Override
    public Kind getKind() {
        return kind;
    }

    @Override
    public Object getValue() {
        return value;
    }

    private static Kind literalKind(final LiteralNode<?> node) {
        if (node.isBoolean()) {
            return Kind.BOOLEAN_LITERAL;
        } else if (node.isNumeric()) {
            return Kind.NUMBER_LITERAL;
        } else if (node.isString()) {
            return Kind.STRING_LITERAL;
        } else if (node.isNull()) {
            return Kind.NULL_LITERAL;
        } else {
            throw new AssertionError("should not reach here: " + node.getValue());
        }
    }

    @Override
    public <R,D> R accept(final TreeVisitor<R,D> visitor, final D data) {
        return visitor.visitLiteral(this, data);
    }
}
