/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.sun.tools.sjavac.options;

import java.util.Iterator;

/**
 *  <p><b>This is NOT part of any supported API.
 *  If you write code that depends on this, you do so at your own risk.
 *  This code and its internal interfaces are subject to change or
 *  deletion without notice.</b>
 */
public class ArgumentIterator implements Iterator<String> {

    /** The underlying argument iterator */
    private Iterator<String> iter;

    /** Extra state used to implement peek and current */
    private String current;
    private String buffered;

    public ArgumentIterator(Iterable<String> iter) {
        this.iter = iter.iterator();
    }

    @Override
    public boolean hasNext() {
        return buffered != null || iter.hasNext();
    }

    @Override
    public String next() {
        fillBuffer();
        current = buffered;
        buffered = null;
        return current;
    }

    /**
     * @return the last element returned by next() (or {@code null} if next has
     * never been invoked on this iterator).
     */
    public String current() {
        return current;
    }

    /** Can't remove current element, since we may have buffered it. */
    @Override
    public void remove() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return Returns the next element without advancing the iterator
     */
    public String peek() {
        fillBuffer();
        return buffered;
    }

    private void fillBuffer() {
        if (buffered == null && iter.hasNext())
            buffered = iter.next();
    }

}
